// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | contact@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The US Standard atmosphere data is taken from the following report: www.dtic.mil/dtic/tr/fulltext/u2/a057506.pdf
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// Object to contain the atmospehric model information
#include "concpressunitsconvert.h"
#include "atmosphere.h"
#include <QMessageBox>
#include <QFile>
#include <QTextStream>
#include <QLocale>
#include <QDir>

Atmosphere::Atmosphere()
{
}

// Method to get and set the path to the folder with the AFGL atmospheric models
QString Atmosphere::getAfglModelsFolderPath() { return this->modelsPath; }
void Atmosphere::setAfglModelsFolderPath(QString newModelsPath) { this->modelsPath = newModelsPath; }

// Method to get the AFGL model by index
OneAtmModel *Atmosphere::getAfglModelByIndex(int index) { return &(this->afglModels[index]); }
OneAtmModel *Atmosphere::getActiveAtmModel() { return &(this->afglModels[this->activeAtmModelNumber]); }

// Methods to get and set the maximum number of molecules in the atmospheric model
int Atmosphere::getMaxNumMols() { return this->maxNumMolecules; }
void Atmosphere::setMaxNumMols(int newMaxNumMols) { this->maxNumMolecules = newMaxNumMols; }

// Function to initialize the AFGL models from external files
void Atmosphere::initAfglModelsFromFiles()
{
    QDir dir;  // Object to read in the directories contained in the current main directory
    QStringList dirList;  // List of sub-directories contained in the current directory
    int i;  // Loop variable


    // Getting a list of model sub-directories in the atmospheric model main directory
    dir.setPath(this->modelsPath);
    dir.setFilter(QDir::AllDirs | QDir::NoDotAndDotDot);
    dirList = dir.entryList();

    // Obtaining a list of directories in the main atmospheric models directory
    afglModels.resize(dirList.count());

    // Reading in atmospehric model information from each directory (for all models available)
    for (i = 0; i < dirList.length(); i++)
    {
        readAtmModel(this->modelsPath + "/" + dirList[i], &(afglModels[i]));
    }
}

// Method to read in the entire atmospheric model from a single model directory
void Atmosphere::readAtmModel(QString dirPath, OneAtmModel *model)
{
    QString tempFileName;  // File name containing the temperature profile
    QString densityFileName;   // File name containing the density profile
    QString totalPressFileName;    // File name containing the total pressure profile
    QStringList partPressFileNames;   // List of file names containing the molecular partial pressure profiles
    QStringList fileList;  // List of files located in the current directory
    QString fileStr;   // String containing a file name or a part of a file name
    QStringList fileSplitStr;  // String containing split portions of a file name
    QDir dir;   // Object to get a list of files in the current directory
    QLocale c;  // Locale string to number conversion
    int j;  // Loop variable
    int index;  // Index of the molecule to be saved into the atmospehric model spectra for a particular molecule


    // Getting a list of model sub-directories in the atmospheric model main directory
    dir.setPath(dirPath);
    dir.setFilter(QDir::Files);
    fileList = dir.entryList();

    // Extracting the model name from one of the files


    // Scanning through all available directories in the current folder
    for (j = 0; j < fileList.count(); j++)
    {
        // Extracting the file identification information from the file name
        fileSplitStr = (fileList[j]).split(".");
        fileStr = fileSplitStr[0];

        // Extracting the sub-portions from the
        fileSplitStr = fileStr.split("-");

        // Appending the files to the list of partial pressure files
        if (fileSplitStr.length() > 2)
        {
            partPressFileNames.append(fileList[j]);
        }
        else
        {
            // Determining the names of the total pressure, temperature and density files
            if ((fileSplitStr[1]).trimmed().toUpper() == "TEMPERATURE" ) { tempFileName = fileList[j]; }
            if ((fileSplitStr[1]).trimmed().toUpper() == "DENSITY" ) { densityFileName = fileList[j]; }
            if ((fileSplitStr[1]).trimmed().toUpper() == "TOTAL PRESS" ) { totalPressFileName = fileList[j]; }
        }
    }

    // Reading in the temperature profile
    readAtmProfileFile(dirPath + "/" + tempFileName, model->getTemp());

    // Reading in the total pressure profile
    readAtmProfileFile(dirPath + "/" + totalPressFileName, model->getTotalPress());

    // Reading in the density profile
    readAtmProfileFile(dirPath + "/" + densityFileName, model->getDensity());

    // Reading in the partial pressures profiles
    model->resizePartPressArray(partPressFileNames.count());
    for (j = 0; j < partPressFileNames.length(); j++)
    {
        fileSplitStr = (partPressFileNames[j]).split("-");
        index = c.toDouble((fileSplitStr[1]).trimmed()) - 1;
        readAtmProfileFile(dirPath + "/" + partPressFileNames[j], model->getPartPressSpecByMolIndex(index));
    }

    // Saving the model name
    fileSplitStr = fileList.first().split("-");
    model->setModelName((fileSplitStr[0]).trimmed());
}

// Method to read in a single atmospheric profile
void Atmosphere::readAtmProfileFile(QString filePath, Spectrum *spec)
{
    QFile inFile(filePath); // Name of the HITRAN file in a text form to use
    QString str;   // File contents
    QStringList strLines;   // String linea read in from a file
    QStringList strValues;  // String representation of values read in from a single line
    QLocale c;     // Object to carry out locale conversions of the string representations into numbers
    int i;    // Loop variable


    // Opening the HITRAN database text file for reading
    inFile.open(QIODevice::ReadOnly);
    QTextStream in(&inFile);

    // Reading all string information from a file
    str = in.readAll();

    // Splitting the string into individual string lines in a file
    strLines = str.split("\n");

    // Reading the first line to establish which column belongs to which molecule and isotope (removing empty lines)
    for (i = 0; i < strLines.length(); i++)
    {
        if ((strLines[i]).count() < 3) { strLines.removeAt(i); }
    }

    //  Splitting the values in a single line read in from a file
    for (i = 0; i < strLines.count(); i++)
    {
        strValues = (strLines[i]).split("\t");
        spec->getY()->append(c.toDouble(strValues[0]));
        spec->getX()->append(c.toDouble(strValues[1]));
    }
}

// Function to initialize the AFGL models from internal data (instead of reading data from files)
void  Atmosphere::initInternalAfglModels()
{
    int i;  // Loop variable
    int j;  // Loop variable
    Spectrum spec;   // Spectrum used to save the read in atmospheric model values
    QVector<double> attenValues;  // Attenuation values
    QVector<double> auxValues;  // Auxiliary values array used to fill out values into another array
    ConcPressUnitsConvert pressConvert;  // Pressure converter


    // Populating the array with the atmospheric models
    afglModels.resize(6);    

    // Extracting and saving the molecule names from the file names

    // Default list of names and numbers of molecules to be used for atmospheric models specification
    //  1:   H2O,   2:    CO2,   3:     O3,   4:   N2O,   5:    CO,   6:   CH4,    7:    O2,    8:    NO,
    //  9:   SO2,  10:    NO2,  11:    NH3,  12:  HNO3,  13:    OH,  14:    HF,   15:   HCL,   16:   HBR,
    // 17:    HI,  18:    CLO,  19:    OCS,  20:  H2CO,  21:  HOCL,  22:    N2,   23:   HCN,   24: CH3CL,
    // 25:  H2O2,  26:   C2H2,  27:   C2H6,  28:   PH3,  29:  COF2,  30:   SF6,   31:   H2S,   32: HCOOH,
    // 33:   HO2,  34:      O,  35: CLONO2,  36:   NO+,  37:  HOBR,  38:   C2H4,  39: CH3OH

    // These profiles are shared between all atmospheric models
    for (i = 0; i < 6; i++)
    {
        // Resizing the arrays with the partial pressure values for each molecule (this maybe should be determined from the model itself)
        afglModels[i].resizePartPressArray(this->maxNumMolecules);

        // 2 - CO2  // Check to make sure that the CO2 profile is also shared
        auxValues = QVector<double> () << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.30E+02 << 3.28E+02 << 3.20E+02 << 3.10E+02 << 2.70E+02 << 1.95E+02 << 1.10E+02 << 6.00E+01 << 4.00E+01 << 3.50E+01;
        afglModels[i].getPartPressSpecByMolIndex(1)->setY(&auxValues);

        // 7 - O2
        auxValues = QVector<double> () << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.09E+05 << 2.00E+05 << 1.90E+05 << 1.80E+05 << 1.60E+05 << 1.40E+05 << 1.20E+05 << 9.40E+04 << 7.25E+04;
        afglModels[i].getPartPressSpecByMolIndex(6)->setY(&auxValues);

        // 8 - NO
        auxValues = QVector<double> () << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 3.00E+04 << 2.99E-04 << 2.95E-04 << 2.83E-04 << 2.68E-04 << 2.52E-04 << 2.40E-04 << 2.44E-04 << 2.55E-04 << 2.77E-04 << 3.07E-04 << 3.60E-04 << 4.51E-04 << 6.85E-04 << 1.28E-03 << 2.45E-03 << 4.53E-03 << 7.14E-03 << 9.34E-03 << 1.12E-02 << 1.19E-02 << 1.17E-02 << 1.10E-02 << 1.03E-02 << 1.01E-02 << 1.01E-02 << 1.03E-02 << 1.15E-02 << 1.61E-02 << 2.68E-02 << 7.01E-02 << 2.13E-01 << 7.12E-01 << 2.08E+00 << 4.50E+00 << 7.98E+00 << 1.00E+01 << 1.00E+01;
        afglModels[i].getPartPressSpecByMolIndex(7)->setY(&auxValues);

        // 9 = SO2
        auxValues = QVector<double> () << 3.00E-04 << 2.74E-04 << 2.36E-04 << 1.90E-04 << 1.46E-04 << 1.18E-04 << 9.71E-05 << 8.30E-05 << 7.21E-05 << 6.56E-05 << 6.08E-05 << 5.79E-05 << 5.60E-05 << 5.59E-05 << 5.64E-05 << 5.75E-05 << 5.75E-05 << 5.37E-05 << 4.78E-05 << 3.97E-05 << 3.19E-05 << 2.67E-05 << 2.28E-05 << 2.07E-05 << 1.90E-05 << 1.75E-05 << 1.54E-05 << 1.34E-05 << 1.21E-05 << 1.16E-05 << 1.21E-05 << 1.36E-05 << 1.65E-05 << 2.10E-05 << 2.77E-05 << 3.56E-05 << 4.59E-05 << 5.15E-05 << 5.11E-05 << 4.32E-05 << 2.83E-05 << 1.33E-05 << 5.56E-06 << 2.24E-06 << 8.96E-07 << 3.58E-07 << 1.43E-07 << 5.73E-08 << 2.29E-08 << 9.17E-09;
        afglModels[i].getPartPressSpecByMolIndex(8)->setY(&auxValues);

        // 10 - NO2
        auxValues = QVector<double> () << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.30E-05 << 2.32E-05 << 2.38E-05 << 2.62E-05 << 3.15E-05 << 4.45E-05 << 7.48E-05 << 1.71E-04 << 3.19E-04 << 5.19E-04 << 7.71E-04 << 1.06E-03 << 1.39E-03 << 1.76E-03 << 2.16E-03 << 2.58E-03 << 3.06E-03 << 3.74E-03 << 4.81E-03 << 6.16E-03 << 7.21E-03 << 7.28E-03 << 6.26E-03 << 4.03E-03 << 2.17E-03 << 1.15E-03 << 6.66E-04 << 4.43E-04 << 3.39E-04 << 2.85E-04 << 2.53E-04 << 2.31E-04 << 2.15E-04 << 2.02E-04 << 1.92E-04 << 1.83E-04 << 1.76E-04 << 1.70E-04 << 1.64E-04 << 1.59E-04 << 1.55E-04 << 1.51E-04;
        afglModels[i].getPartPressSpecByMolIndex(9)->setY(&auxValues);

        // 11 - NH3
        auxValues = QVector<double> () << 5.00E-04 << 5.00E-04 << 4.63E-04 << 3.80E-04 << 2.88E-04 << 2.04E-04 << 1.46E-04 << 9.88E-05 << 6.48E-05 << 3.77E-05 << 2.03E-05 << 1.09E-05 << 6.30E-06 << 3.12E-06 << 1.11E-06 << 4.47E-07 << 2.11E-07 << 1.10E-07 << 6.70E-08 << 3.97E-08 << 2.41E-08 << 1.92E-08 << 1.72E-08 << 1.59E-08 << 1.44E-08 << 1.23E-08 << 9.37E-09 << 6.35E-09 << 3.68E-09 << 1.82E-09 << 9.26E-10 << 2.94E-10 << 8.72E-11 << 2.98E-11 << 1.30E-11 << 7.13E-12 << 4.80E-12 << 3.66E-12 << 3.00E-12 << 2.57E-12 << 2.27E-12 << 2.04E-12 << 1.85E-12 << 1.71E-12 << 1.59E-12 << 1.48E-12 << 1.40E-12 << 1.32E-12 << 1.25E-12 << 1.19E-12;
        afglModels[i].getPartPressSpecByMolIndex(10)->setY(&auxValues);

        // 12 - HNO3
        auxValues = QVector<double> () << 5.00E-05 << 5.96E-05 << 6.93E-05 << 7.91E-05 << 8.87E-05 << 9.75E-05 << 1.11E-04 << 1.26E-04 << 1.39E-04 << 1.53E-04 << 1.74E-04 << 2.02E-04 << 2.41E-04 << 2.76E-04 << 3.33E-04 << 4.52E-04 << 7.37E-04 << 1.31E-03 << 2.11E-03 << 3.17E-03 << 4.20E-03 << 4.94E-03 << 5.46E-03 << 5.74E-03 << 5.84E-03 << 5.61E-03 << 4.82E-03 << 3.74E-03 << 2.59E-03 << 1.64E-03 << 9.68E-04 << 5.33E-04 << 2.52E-04 << 1.21E-04 << 7.70E-05 << 5.55E-05 << 4.45E-05 << 3.84E-05 << 3.49E-05 << 3.27E-05 << 3.12E-05 << 3.01E-05 << 2.92E-05 << 2.84E-05 << 2.78E-05 << 2.73E-05 << 2.68E-05 << 2.64E-05 << 2.60E-05 << 2.57E-05;
        afglModels[i].getPartPressSpecByMolIndex(11)->setY(&auxValues);

        // 13 - OH
        auxValues = QVector<double> () << 4.40E-08 << 4.40E-08 << 4.40E-08 << 4.40E-08 << 4.40E-08 << 4.40E-08 << 4.40E-08 << 4.41E-08 << 4.45E-08 << 4.56E-08 << 4.68E-08 << 4.80E-08 << 4.94E-08 << 5.19E-08 << 5.65E-08 << 6.75E-08 << 8.25E-08 << 1.04E-07 << 1.30E-07 << 1.64E-07 << 2.16E-07 << 3.40E-07 << 5.09E-07 << 7.59E-07 << 1.16E-06 << 2.18E-06 << 5.00E-06 << 1.17E-05 << 3.40E-05 << 8.35E-05 << 1.70E-04 << 2.85E-04 << 4.06E-04 << 5.11E-04 << 5.79E-04 << 6.75E-04 << 9.53E-04 << 1.76E-03 << 3.74E-03 << 7.19E-03 << 1.12E-02 << 1.13E-02 << 6.10E-03 << 1.51E-03 << 2.42E-04 << 4.47E-05 << 1.77E-05 << 1.19E-05 << 1.35E-05 << 2.20E-05;
        afglModels[i].getPartPressSpecByMolIndex(12)->setY(&auxValues);

        // 14 - HF
        auxValues = QVector<double> () << 1.00E-08 << 1.00E-08 << 1.23E-08 << 1.97E-08 << 3.18E-08 << 5.63E-08 << 9.18E-08 << 1.53E-07 << 2.41E-07 << 4.04E-07 << 6.57E-07 << 1.20E-06 << 1.96E-06 << 3.12E-06 << 4.62E-06 << 7.09E-06 << 1.05E-05 << 1.69E-05 << 2.57E-05 << 4.02E-05 << 5.77E-05 << 7.77E-05 << 9.90E-05 << 1.23E-04 << 1.50E-04 << 1.82E-04 << 2.30E-04 << 2.83E-04 << 3.20E-04 << 3.48E-04 << 3.72E-04 << 3.95E-04 << 4.10E-04 << 4.21E-04 << 4.24E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04 << 4.25E-04;
        afglModels[i].getPartPressSpecByMolIndex(13)->setY(&auxValues);

        // 15 - HCL
        auxValues = QVector<double> () << 1.00E-03 << 7.49E-04 << 5.61E-04 << 4.22E-04 << 3.19E-04 << 2.39E-04 << 1.79E-04 << 1.32E-04 << 9.96E-05 << 7.48E-05 << 5.68E-05 << 4.59E-05 << 4.36E-05 << 6.51E-05 << 1.01E-04 << 1.63E-04 << 2.37E-04 << 3.13E-04 << 3.85E-04 << 4.42E-04 << 4.89E-04 << 5.22E-04 << 5.49E-04 << 5.75E-04 << 6.04E-04 << 6.51E-04 << 7.51E-04 << 9.88E-04 << 1.28E-03 << 1.57E-03 << 1.69E-03 << 1.74E-03 << 1.76E-03 << 1.79E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03 << 1.80E-03;
        afglModels[i].getPartPressSpecByMolIndex(14)->setY(&auxValues);

        // 16 - HBr
        auxValues = QVector<double> () << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.70E-06 << 1.71E-06 << 1.76E-06 << 1.90E-06 << 2.26E-06 << 2.82E-06 << 3.69E-06 << 4.91E-06 << 6.13E-06 << 6.85E-06 << 7.08E-06 << 7.14E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06 << 7.15E-06;
        afglModels[i].getPartPressSpecByMolIndex(15)->setY(&auxValues);

        // 17 - HI
        auxValues = QVector<double> () << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06 << 3.00E-06;
        afglModels[i].getPartPressSpecByMolIndex(16)->setY(&auxValues);

        // 18 - ClO
        auxValues = QVector<double> () << 1.00E-08 << 1.00E-08 << 1.00E-08 << 1.00E-08 << 1.00E-08 << 1.00E-08 << 1.00E-08 << 1.00E-08 << 1.01E-08 << 1.05E-08 << 1.21E-08 << 1.87E-08 << 3.18E-08 << 5.61E-08 << 9.99E-08 << 1.78E-07 << 3.16E-07 << 5.65E-07 << 1.04E-06 << 2.04E-06 << 4.64E-06 << 8.15E-06 << 1.07E-05 << 1.52E-05 << 2.24E-05 << 3.97E-05 << 8.48E-05 << 1.85E-04 << 3.57E-04 << 5.08E-04 << 6.07E-04 << 5.95E-04 << 4.33E-04 << 2.51E-04 << 1.56E-04 << 1.04E-04 << 7.69E-05 << 6.30E-05 << 5.52E-05 << 5.04E-05 << 4.72E-05 << 4.49E-05 << 4.30E-05 << 4.16E-05 << 4.03E-05 << 3.93E-05 << 3.83E-05 << 3.75E-05 << 3.68E-05 << 3.61E-05;
        afglModels[i].getPartPressSpecByMolIndex(17)->setY(&auxValues);

        // 19 - OCS
        auxValues = QVector<double> () << 6.00E-04 << 5.90E-04 << 5.80E-04 << 5.70E-04 << 5.62E-04 << 5.55E-04 << 5.48E-04 << 5.40E-04 << 5.32E-04 << 5.25E-04 << 5.18E-04 << 5.09E-04 << 4.98E-04 << 4.82E-04 << 4.60E-04 << 4.26E-04 << 3.88E-04 << 3.48E-04 << 3.09E-04 << 2.74E-04 << 2.41E-04 << 2.14E-04 << 1.88E-04 << 1.64E-04 << 1.37E-04 << 1.08E-04 << 6.70E-05 << 2.96E-05 << 1.21E-05 << 4.31E-06 << 1.60E-06 << 6.71E-07 << 4.35E-07 << 3.34E-07 << 2.80E-07 << 2.47E-07 << 2.28E-07 << 2.16E-07 << 2.08E-07 << 2.03E-07 << 1.98E-07 << 1.95E-07 << 1.92E-07 << 1.89E-07 << 1.87E-07 << 1.85E-07 << 1.83E-07 << 1.81E-07 << 1.80E-07 << 1.78E-07;
        afglModels[i].getPartPressSpecByMolIndex(18)->setY(&auxValues);

        // 20 - H2CO
        auxValues = QVector<double> () << 2.40E-03 << 1.07E-03 << 4.04E-04 << 2.27E-04 << 1.40E-04 << 1.00E-04 << 7.44E-05 << 6.04E-05 << 5.01E-05 << 4.22E-05 << 3.63E-05 << 3.43E-05 << 3.39E-05 << 3.50E-05 << 3.62E-05 << 3.62E-05 << 3.58E-05 << 3.50E-05 << 3.42E-05 << 3.39E-05 << 3.43E-05 << 3.68E-05 << 4.03E-05 << 4.50E-05 << 5.06E-05 << 5.82E-05 << 7.21E-05 << 8.73E-05 << 1.01E-04 << 1.11E-04 << 1.13E-04 << 1.03E-04 << 7.95E-05 << 4.82E-05 << 1.63E-05 << 5.10E-06 << 2.00E-06 << 1.05E-06 << 6.86E-07 << 5.14E-07 << 4.16E-07 << 3.53E-07 << 3.09E-07 << 2.76E-07 << 2.50E-07 << 2.30E-07 << 2.13E-07 << 1.98E-07 << 1.86E-07 << 1.75E-07;
        afglModels[i].getPartPressSpecByMolIndex(19)->setY(&auxValues);

        // 21 - HOCl
        auxValues = QVector<double> () << 7.70E-06 << 1.06E-05 << 1.22E-05 << 1.14E-05 << 9.80E-06 << 8.01E-06 << 6.42E-06 << 5.42E-06 << 4.70E-06 << 4.41E-06 << 4.34E-06 << 4.65E-06 << 5.01E-06 << 5.22E-06 << 5.60E-06 << 6.86E-06 << 8.77E-06 << 1.20E-05 << 1.63E-05 << 2.26E-05 << 3.07E-05 << 4.29E-05 << 5.76E-05 << 7.65E-05 << 9.92E-05 << 1.31E-04 << 1.84E-04 << 2.45E-04 << 2.96E-04 << 3.21E-04 << 3.04E-04 << 2.48E-04 << 1.64E-04 << 9.74E-05 << 4.92E-05 << 2.53E-05 << 1.50E-05 << 1.05E-05 << 8.34E-06 << 7.11E-06 << 6.33E-06 << 5.78E-06 << 5.37E-06 << 5.05E-06 << 4.78E-06 << 4.56E-06 << 4.37E-06 << 4.21E-06 << 4.06E-06 << 3.93E-06;
        afglModels[i].getPartPressSpecByMolIndex(20)->setY(&auxValues);

        // 22 - N2
        auxValues = QVector<double> () << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.81E+05 << 7.80E+05 << 7.79E+05 << 7.77E+05 << 7.74E+05 << 7.70E+05 << 7.65E+05 << 7.60E+05;
        afglModels[i].getPartPressSpecByMolIndex(21)->setY(&auxValues);

        // 23 - HCN
        auxValues = QVector<double> () << 1.70E-04 << 1.65E-04 << 1.63E-04 << 1.61E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.60E-04 << 1.59E-04 << 1.57E-04 << 1.55E-04 << 1.52E-04 << 1.49E-04 << 1.45E-04 << 1.41E-04 << 1.37E-04 << 1.34E-04 << 1.30E-04 << 1.25E-04 << 1.19E-04 << 1.13E-04 << 1.05E-04 << 9.73E-05 << 9.04E-05 << 8.46E-05 << 8.02E-05 << 7.63E-05 << 7.30E-05 << 7.00E-05 << 6.70E-05 << 6.43E-05 << 6.21E-05 << 6.02E-05 << 5.88E-05 << 5.75E-05 << 5.62E-05 << 5.50E-05 << 5.37E-05 << 5.25E-05 << 5.12E-05 << 5.00E-05 << 4.87E-05 << 4.75E-05 << 4.62E-05 << 4.50E-05;
        afglModels[i].getPartPressSpecByMolIndex(22)->setY(&auxValues);

        // 24 - CH3Cl
        auxValues = QVector<double> () << 7.00E-04 << 6.70E-04 << 6.43E-04 << 6.22E-04 << 6.07E-04 << 6.02E-04 << 6.00E-04 << 6.00E-04 << 5.98E-04 << 5.94E-04 << 5.88E-04 << 5.79E-04 << 5.66E-04 << 5.48E-04 << 5.28E-04 << 5.03E-04 << 4.77E-04 << 4.49E-04 << 4.21E-04 << 3.95E-04 << 3.69E-04 << 3.43E-04 << 3.17E-04 << 2.86E-04 << 2.48E-04 << 1.91E-04 << 1.10E-04 << 4.72E-05 << 1.79E-05 << 7.35E-06 << 3.03E-06 << 1.32E-06 << 8.69E-07 << 6.68E-07 << 5.60E-07 << 4.94E-07 << 4.56E-07 << 4.32E-07 << 4.17E-07 << 4.05E-07 << 3.96E-07 << 3.89E-07 << 3.83E-07 << 3.78E-07 << 3.73E-07 << 3.69E-07 << 3.66E-07 << 3.62E-07 << 3.59E-07 << 3.56E-07;
        afglModels[i].getPartPressSpecByMolIndex(23)->setY(&auxValues);

        // 25 - H2O2
        auxValues = QVector<double> () << 2.00E-04 << 1.95E-04 << 1.92E-04 << 1.89E-04 << 1.84E-04 << 1.77E-04 << 1.66E-04 << 1.49E-04 << 1.23E-04 << 9.09E-05 << 5.79E-05 << 3.43E-05 << 1.95E-05 << 1.08E-05 << 6.59E-06 << 4.20E-06 << 2.94E-06 << 2.30E-06 << 2.24E-06 << 2.68E-06 << 3.68E-06 << 5.62E-06 << 1.03E-05 << 1.97E-05 << 3.70E-05 << 6.20E-05 << 1.03E-04 << 1.36E-04 << 1.36E-04 << 1.13E-04 << 8.51E-05 << 6.37E-05 << 5.17E-05 << 4.44E-05 << 3.80E-05 << 3.48E-05 << 3.62E-05 << 5.25E-05 << 1.26E-04 << 3.77E-04 << 1.12E-03 << 2.00E-03 << 1.68E-03 << 4.31E-04 << 4.98E-05 << 6.76E-06 << 8.38E-07 << 9.56E-08 << 1.00E-08 << 1.00E-09;
        afglModels[i].getPartPressSpecByMolIndex(24)->setY(&auxValues);

        // 26 - C2H2
        auxValues = QVector<double> () << 3.00E-04 << 1.72E-04 << 9.57E-05 << 6.74E-05 << 5.07E-05 << 3.99E-05 << 3.19E-05 << 2.80E-05 << 2.55E-05 << 2.40E-05 << 2.27E-05 << 2.08E-05 << 1.76E-05 << 1.23E-05 << 7.32E-06 << 4.52E-06 << 2.59E-06 << 1.55E-06 << 8.63E-07 << 5.30E-07 << 3.10E-07 << 1.89E-07 << 1.04E-07 << 5.75E-08 << 2.23E-08 << 8.51E-09 << 4.09E-09 << 2.52E-09 << 1.86E-09 << 1.52E-09 << 1.32E-09 << 1.18E-09 << 1.08E-09 << 9.97E-10 << 9.34E-10 << 8.83E-10 << 8.43E-10 << 8.10E-10 << 7.83E-10 << 7.60E-10 << 7.40E-10 << 7.23E-10 << 7.07E-10 << 6.94E-10 << 6.81E-10 << 6.70E-10 << 6.59E-10 << 6.49E-10 << 6.40E-10 << 6.32E-10;
        afglModels[i].getPartPressSpecByMolIndex(25)->setY(&auxValues);

        // 27 - C2H6
        auxValues = QVector<double> () << 2.00E-03 << 2.00E-03 << 2.00E-03 << 2.00E-03 << 1.98E-03 << 1.95E-03 << 1.90E-03 << 1.85E-03 << 1.79E-03 << 1.72E-03 << 1.58E-03 << 1.30E-03 << 9.86E-04 << 7.22E-04 << 4.96E-04 << 3.35E-04 << 2.14E-04 << 1.49E-04 << 1.05E-04 << 7.96E-05 << 6.01E-05 << 4.57E-05 << 3.40E-05 << 2.60E-05 << 1.89E-05 << 1.22E-05 << 5.74E-06 << 2.14E-06 << 8.49E-07 << 3.42E-07 << 1.34E-07 << 5.39E-08 << 2.25E-08 << 1.04E-08 << 6.57E-09 << 4.74E-09 << 3.79E-09 << 3.28E-09 << 2.98E-09 << 2.79E-09 << 2.66E-09 << 2.56E-09 << 2.49E-09 << 2.43E-09 << 2.37E-09 << 2.33E-09 << 2.29E-09 << 2.25E-09 << 2.22E-09 << 2.19E-09;
        afglModels[i].getPartPressSpecByMolIndex(26)->setY(&auxValues);

        // 28 - PH3
        auxValues = QVector<double> () << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14 << 1.00E-14;
        afglModels[i].getPartPressSpecByMolIndex(27)->setY(&auxValues);
    }

    // Filling out the concentrations for unavailable molecules and the altitude grid values
    for (i = 0; i < afglModels.count(); i++)
    {
        // The rest of the molecules are not in the US Standard models
        for (j = 28; j < this->maxNumMolecules; j++)
        {
            auxValues = QVector<double> () << 1E-18 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 << 0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0 <<  0;
            afglModels[i].getPartPressSpecByMolIndex(j)->setY(&auxValues);
        }
        // Saving the altitude values into the model data
        for (j = 0; j < this->maxNumMolecules; j++)
        {
            auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
            afglModels[i].getPartPressSpecByMolIndex(j)->setX(&auxValues);
        }
    }


    // 1 - Tropical atmospheric model
    // 1 - H2O
    auxValues = QVector<double> () << 2.59E+04 << 1.95E+04 << 1.53E+04 << 8.60E+03 << 4.44E+03 << 3.35E+03 << 2.10E+03 << 1.29E+03 << 7.64E+02 << 4.10E+02 << 1.91E+02 << 7.31E+01 << 2.91E+01 << 9.90E+00 << 6.22E+00 << 4.00E+00 << 3.00E+00 << 2.90E+00 << 2.75E+00 << 2.60E+00 << 2.60E+00 << 2.65E+00 << 2.80E+00 << 2.90E+00 << 3.20E+00 << 3.25E+00 << 3.60E+00 << 4.00E+00 << 4.30E+00 << 4.60E+00 << 4.90E+00 << 5.20E+00 << 5.50E+00 << 5.70E+00 << 5.90E+00 << 6.00E+00 << 6.00E+00 << 6.00E+00 << 5.40E+00 << 4.50E+00 << 3.30E+00 << 2.10E+00 << 1.30E+00 << 8.50E-01 << 5.40E-01 << 4.00E-01 << 3.40E-01 << 2.80E-01 << 2.40E-01 << 2.00E-01;
    afglModels[0].getPartPressSpecByMolIndex(0)->setY(&auxValues);
    // 3 - O3
    auxValues = QVector<double> () << 2.87E-02 << 3.15E-02 << 3.34E-02 << 3.50E-02 << 3.56E-02 << 3.77E-02 << 3.99E-02 << 4.22E-02 << 4.47E-02 << 5.00E-02 << 5.60E-02 << 6.61E-02 << 7.82E-02 << 9.29E-02 << 1.05E-01 << 1.26E-01 << 1.44E-01 << 2.50E-01 << 5.00E-01 << 9.50E-01 << 1.40E+00 << 1.80E+00 << 2.40E+00 << 3.40E+00 << 4.30E+00 << 5.40E+00 << 7.80E+00 << 9.30E+00 << 9.85E+00 << 9.70E+00 << 8.80E+00 << 7.50E+00 << 5.90E+00 << 4.50E+00 << 3.45E+00 << 2.80E+00 << 1.80E+00 << 1.10E+00 << 6.50E-01 << 3.00E-01 << 1.80E-01 << 3.30E-01 << 5.00E-01 << 5.20E-01 << 5.00E-01 << 4.00E-01 << 2.00E-01 << 5.00E-02 << 5.00E-03 << 5.00E-04;
    afglModels[0].getPartPressSpecByMolIndex(2)->setY(&auxValues);
    // 4 - N2O
    auxValues = QVector<double> () << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.18E-01 << 3.14E-01 << 3.10E-01 << 3.05E-01 << 3.00E-01 << 2.94E-01 << 2.88E-01 << 2.78E-01 << 2.67E-01 << 2.53E-01 << 2.37E-01 << 2.19E-01 << 2.05E-01 << 1.97E-01 << 1.88E-01 << 1.76E-01 << 1.59E-01 << 1.42E-01 << 1.17E-01 << 9.28E-02 << 6.69E-02 << 4.51E-02 << 2.75E-02 << 1.59E-02 << 9.38E-03 << 4.75E-03 << 3.00E-03 << 2.07E-03 << 1.51E-03 << 1.15E-03 << 8.89E-04 << 7.06E-04 << 5.72E-04 << 4.71E-04 << 3.93E-04 << 3.32E-04 << 2.84E-04 << 2.44E-04 << 2.12E-04 << 1.85E-04;
    afglModels[0].getPartPressSpecByMolIndex(3)->setY(&auxValues);
    // 5 - CO
    auxValues = QVector<double> () << 1.50E-01 << 1.45E-01 << 1.40E-01 << 1.35E-01 << 1.31E-01 << 1.30E-01 << 1.29E-01 << 1.25E-01 << 1.19E-01 << 1.09E-01 << 9.96E-02 << 8.96E-02 << 7.81E-02 << 6.37E-02 << 5.03E-02 << 3.94E-02 << 3.07E-02 << 2.49E-02 << 1.97E-02 << 1.55E-02 << 1.33E-02 << 1.23E-02 << 1.23E-02 << 1.31E-02 << 1.40E-02 << 1.52E-02 << 1.72E-02 << 2.00E-02 << 2.27E-02 << 2.49E-02 << 2.74E-02 << 3.10E-02 << 3.51E-02 << 3.99E-02 << 4.48E-02 << 5.09E-02 << 5.99E-02 << 6.96E-02 << 9.19E-02 << 1.94E-01 << 5.69E-01 << 1.55E+00 << 3.85E+00 << 6.59E+00 << 1.04E+01 << 1.71E+01 << 2.47E+01 << 3.36E+01 << 4.15E+01 << 5.00E+01;
    afglModels[0].getPartPressSpecByMolIndex(4)->setY(&auxValues);
    // 6 - CH4
    auxValues = QVector<double> () << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.69E+00 << 1.69E+00 << 1.68E+00 << 1.66E+00 << 1.65E+00 << 1.63E+00 << 1.61E+00 << 1.58E+00 << 1.55E+00 << 1.52E+00 << 1.48E+00 << 1.42E+00 << 1.36E+00 << 1.27E+00 << 1.19E+00 << 1.12E+00 << 1.06E+00 << 9.87E-01 << 9.14E-01 << 8.30E-01 << 7.46E-01 << 6.62E-01 << 5.64E-01 << 4.61E-01 << 3.63E-01 << 2.77E-01 << 2.10E-01 << 1.65E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.40E-01 << 1.30E-01 << 1.20E-01 << 1.10E-01 << 9.50E-02 << 6.00E-02 << 3.00E-02;
    afglModels[0].getPartPressSpecByMolIndex(5)->setY(&auxValues);
    // Other model parameters (Total pressure, Temeprature, Atmosperhic density)
    auxValues = QVector<double> () << 1.01E+03 << 9.04E+02 << 8.05E+02 << 7.15E+02 << 6.33E+02 << 5.59E+02 << 4.92E+02 << 4.32E+02 << 3.78E+02 << 3.29E+02 << 2.86E+02 << 2.47E+02 << 2.13E+02 << 1.82E+02 << 1.56E+02 << 1.32E+02 << 1.11E+02 << 9.37E+01 << 7.89E+01 << 6.66E+01 << 5.65E+01 << 4.80E+01 << 4.09E+01 << 3.50E+01 << 3.00E+01 << 2.57E+01 << 1.76E+01 << 1.22E+01 << 8.52E+00 << 6.00E+00 << 4.26E+00 << 3.05E+00 << 2.20E+00 << 1.59E+00 << 1.16E+00 << 8.54E-01 << 4.56E-01 << 2.39E-01 << 1.21E-01 << 5.80E-02 << 2.60E-02 << 1.10E-02 << 4.40E-03 << 1.72E-03 << 6.88E-04 << 2.89E-04 << 1.30E-04 << 6.47E-05 << 3.60E-05 << 2.25E-05;
    afglModels[0].getTotalPress()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[0].getTotalPress()->setX(&auxValues);
    auxValues = QVector<double> () << 3.00E+02 << 2.94E+02 << 2.88E+02 << 2.84E+02 << 2.77E+02 << 2.70E+02 << 2.64E+02 << 2.57E+02 << 2.50E+02 << 2.44E+02 << 2.37E+02 << 2.30E+02 << 2.24E+02 << 2.17E+02 << 2.10E+02 << 2.04E+02 << 1.97E+02 << 1.95E+02 << 1.99E+02 << 2.03E+02 << 2.07E+02 << 2.11E+02 << 2.15E+02 << 2.17E+02 << 2.19E+02 << 2.21E+02 << 2.27E+02 << 2.32E+02 << 2.38E+02 << 2.43E+02 << 2.49E+02 << 2.54E+02 << 2.59E+02 << 2.65E+02 << 2.70E+02 << 2.70E+02 << 2.63E+02 << 2.53E+02 << 2.36E+02 << 2.19E+02 << 2.02E+02 << 1.85E+02 << 1.77E+02 << 1.77E+02 << 1.84E+02 << 1.91E+02 << 2.12E+02 << 2.42E+02 << 3.00E+02 << 3.80E+02;
    afglModels[0].getTemp()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[0].getTemp()->setX(&auxValues);
    auxValues = QVector<double> () << 2.45E+19 << 2.23E+19 << 2.03E+19 << 1.83E+19 << 1.66E+19 << 1.50E+19 << 1.35E+19 << 1.22E+19 << 1.10E+19 << 9.79E+18 << 8.75E+18 << 7.78E+18 << 6.90E+18 << 6.08E+18 << 5.38E+18 << 4.70E+18 << 4.08E+18 << 3.49E+18 << 2.88E+18 << 2.38E+18 << 1.98E+18 << 1.65E+18 << 1.38E+18 << 1.17E+18 << 9.92E+17 << 8.41E+17 << 5.63E+17 << 3.81E+17 << 2.60E+17 << 1.79E+17 << 1.24E+17 << 8.70E+16 << 6.15E+16 << 4.35E+16 << 3.12E+16 << 2.29E+16 << 1.26E+16 << 6.84E+15 << 3.72E+15 << 1.92E+15 << 9.34E+14 << 4.31E+14 << 1.80E+14 << 7.04E+13 << 2.71E+13 << 1.10E+13 << 4.45E+12 << 1.94E+12 << 8.71E+11 << 4.23E+11;
    afglModels[0].getDensity()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[0].getDensity()->setX(&auxValues);
    afglModels[0].setModelName("Tropical");


    // 2 - Midlatitude summer
    // 1 - H2O
    auxValues = QVector<double> () << 1.88E+04 << 1.38E+04 << 9.68E+03 << 5.98E+03 << 3.81E+03 << 2.23E+03 << 1.51E+03 << 1.02E+03 << 6.46E+02 << 4.13E+02 << 2.47E+02 << 9.56E+01 << 2.94E+01 << 8.00E+00 << 5.00E+00 << 3.40E+00 << 3.30E+00 << 3.20E+00 << 3.15E+00 << 3.20E+00 << 3.30E+00 << 3.45E+00 << 3.60E+00 << 3.85E+00 << 4.00E+00 << 4.20E+00 << 4.45E+00 << 4.70E+00 << 4.85E+00 << 4.95E+00 << 5.00E+00 << 5.10E+00 << 5.30E+00 << 5.45E+00 << 5.50E+00 << 5.50E+00 << 5.35E+00 << 5.00E+00 << 4.40E+00 << 3.70E+00 << 2.95E+00 << 2.10E+00 << 1.33E+00 << 8.50E-01 << 5.40E-01 << 4.00E-01 << 3.40E-01 << 2.80E-01 << 2.40E-01 << 2.00E-01;
    afglModels[1].getPartPressSpecByMolIndex(0)->setY(&auxValues);
    // 3 - O3
    auxValues = QVector<double> () << 3.02E-02 << 3.34E-02 << 3.69E-02 << 4.22E-02 << 4.82E-02 << 5.51E-02 << 6.41E-02 << 7.76E-02 << 9.13E-02 << 1.11E-01 << 1.30E-01 << 1.79E-01 << 2.23E-01 << 3.00E-01 << 4.40E-01 << 5.00E-01 << 6.00E-01 << 7.00E-01 << 1.00E+00 << 1.50E+00 << 2.00E+00 << 2.40E+00 << 2.90E+00 << 3.40E+00 << 4.00E+00 << 4.80E+00 << 6.00E+00 << 7.00E+00 << 8.10E+00 << 8.90E+00 << 8.70E+00 << 7.55E+00 << 5.90E+00 << 4.50E+00 << 3.50E+00 << 2.80E+00 << 1.80E+00 << 1.30E+00 << 8.00E-01 << 4.00E-01 << 1.90E-01 << 2.00E-01 << 5.70E-01 << 7.50E-01 << 7.00E-01 << 4.00E-01 << 2.00E-01 << 5.00E-02 << 5.00E-03 << 5.00E-04;
    afglModels[1].getPartPressSpecByMolIndex(2)->setY(&auxValues);
    // 4 - N2O
    auxValues = QVector<double> () << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.16E-01 << 3.10E-01 << 2.99E-01 << 2.94E-01 << 2.86E-01 << 2.80E-01 << 2.72E-01 << 2.61E-01 << 2.42E-01 << 2.17E-01 << 1.84E-01 << 1.61E-01 << 1.32E-01 << 1.15E-01 << 1.04E-01 << 9.62E-02 << 8.96E-02 << 8.01E-02 << 6.70E-02 << 4.96E-02 << 3.70E-02 << 2.52E-02 << 1.74E-02 << 1.16E-02 << 7.67E-03 << 5.32E-03 << 3.22E-03 << 2.03E-03 << 1.40E-03 << 1.02E-03 << 7.77E-04 << 6.26E-04 << 5.17E-04 << 4.35E-04 << 3.73E-04 << 3.24E-04 << 2.84E-04 << 2.52E-04 << 2.26E-04 << 2.04E-04 << 1.85E-04;
    afglModels[1].getPartPressSpecByMolIndex(3)->setY(&auxValues);
    // 5 - CO
    auxValues = QVector<double> () << 1.50E-01 << 1.45E-01 << 1.40E-01 << 1.35E-01 << 1.31E-01 << 1.30E-01 << 1.29E-01 << 1.25E-01 << 1.19E-01 << 1.09E-01 << 9.96E-02 << 8.96E-02 << 7.81E-02 << 6.37E-02 << 5.03E-02 << 3.94E-02 << 3.07E-02 << 2.49E-02 << 1.97E-02 << 1.55E-02 << 1.33E-02 << 1.23E-02 << 1.23E-02 << 1.31E-02 << 1.40E-02 << 1.52E-02 << 1.72E-02 << 2.00E-02 << 2.27E-02 << 2.49E-02 << 2.72E-02 << 2.96E-02 << 3.14E-02 << 3.31E-02 << 3.49E-02 << 3.65E-02 << 3.92E-02 << 4.67E-02 << 6.40E-02 << 1.18E-01 << 2.94E-01 << 6.82E-01 << 1.47E+00 << 2.85E+00 << 5.17E+00 << 1.01E+01 << 1.87E+01 << 2.86E+01 << 3.89E+01 << 5.00E+01;
    afglModels[1].getPartPressSpecByMolIndex(4)->setY(&auxValues);
    // 6 - CH4
    auxValues = QVector<double> () << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.69E+00 << 1.67E+00 << 1.65E+00 << 1.63E+00 << 1.62E+00 << 1.58E+00 << 1.54E+00 << 1.51E+00 << 1.48E+00 << 1.45E+00 << 1.42E+00 << 1.39E+00 << 1.36E+00 << 1.32E+00 << 1.28E+00 << 1.22E+00 << 1.15E+00 << 1.07E+00 << 9.73E-01 << 8.80E-01 << 7.89E-01 << 7.05E-01 << 6.32E-01 << 5.59E-01 << 5.01E-01 << 4.45E-01 << 3.92E-01 << 3.39E-01 << 2.87E-01 << 2.38E-01 << 1.94E-01 << 1.57E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.40E-01 << 1.30E-01 << 1.20E-01 << 1.10E-01 << 9.50E-02 << 6.00E-02 << 3.00E-02;
    afglModels[1].getPartPressSpecByMolIndex(5)->setY(&auxValues);
    // Other model parameters (Total pressure, Temeprature, Atmosperhic density)
    auxValues = QVector<double> () << 1.01E+03 << 9.02E+02 << 8.02E+02 << 7.10E+02 << 6.28E+02 << 5.54E+02 << 4.87E+02 << 4.26E+02 << 3.72E+02 << 3.24E+02 << 2.81E+02 << 2.43E+02 << 2.09E+02 << 1.79E+02 << 1.53E+02 << 1.30E+02 << 1.11E+02 << 9.50E+01 << 8.12E+01 << 6.95E+01 << 5.95E+01 << 5.10E+01 << 4.37E+01 << 3.76E+01 << 3.22E+01 << 2.77E+01 << 1.91E+01 << 1.32E+01 << 9.30E+00 << 6.52E+00 << 4.64E+00 << 3.33E+00 << 2.41E+00 << 1.76E+00 << 1.29E+00 << 9.51E-01 << 5.15E-01 << 2.72E-01 << 1.39E-01 << 6.70E-02 << 3.00E-02 << 1.20E-02 << 4.48E-03 << 1.64E-03 << 6.25E-04 << 2.58E-04 << 1.17E-04 << 6.11E-05 << 3.56E-05 << 2.27E-05;
    afglModels[1].getTotalPress()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[1].getTotalPress()->setX(&auxValues);
    auxValues = QVector<double> () << 2.94E+02 << 2.90E+02 << 2.85E+02 << 2.79E+02 << 2.73E+02 << 2.67E+02 << 2.61E+02 << 2.55E+02 << 2.48E+02 << 2.42E+02 << 2.35E+02 << 2.29E+02 << 2.22E+02 << 2.16E+02 << 2.16E+02 << 2.16E+02 << 2.16E+02 << 2.16E+02 << 2.17E+02 << 2.18E+02 << 2.19E+02 << 2.20E+02 << 2.22E+02 << 2.23E+02 << 2.24E+02 << 2.25E+02 << 2.29E+02 << 2.34E+02 << 2.39E+02 << 2.45E+02 << 2.51E+02 << 2.58E+02 << 2.64E+02 << 2.70E+02 << 2.75E+02 << 2.76E+02 << 2.69E+02 << 2.57E+02 << 2.40E+02 << 2.18E+02 << 1.96E+02 << 1.74E+02 << 1.65E+02 << 1.65E+02 << 1.78E+02 << 1.91E+02 << 2.22E+02 << 2.62E+02 << 3.17E+02 << 3.80E+02;
    afglModels[1].getTemp()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[1].getTemp()->setX(&auxValues);
    auxValues = QVector<double> () << 2.50E+19 << 2.26E+19 << 2.04E+19 << 1.84E+19 << 1.67E+19 << 1.50E+19 << 1.35E+19 << 1.21E+19 << 1.09E+19 << 9.72E+18 << 8.66E+18 << 7.70E+18 << 6.81E+18 << 6.01E+18 << 5.14E+18 << 4.37E+18 << 3.73E+18 << 3.19E+18 << 2.72E+18 << 2.31E+18 << 1.97E+18 << 1.68E+18 << 1.43E+18 << 1.22E+18 << 1.04E+18 << 8.92E+17 << 6.05E+17 << 4.09E+17 << 2.82E+17 << 1.93E+17 << 1.34E+17 << 9.37E+16 << 6.62E+16 << 4.73E+16 << 3.40E+16 << 2.50E+16 << 1.39E+16 << 7.67E+15 << 4.20E+15 << 2.23E+15 << 1.11E+15 << 5.00E+14 << 1.97E+14 << 7.20E+13 << 2.54E+13 << 9.82E+12 << 3.82E+12 << 1.69E+12 << 8.15E+11 << 4.33E+11;
    afglModels[1].getDensity()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[1].getDensity()->setX(&auxValues);
    afglModels[1].setModelName("Midlatitude summer");


    // 3 -  Midlatitude winter
    // 1 - H2O
    auxValues = QVector<double> () << 4.32E+03 << 3.45E+03 << 2.79E+03 << 2.09E+03 << 1.28E+03 << 8.24E+02 << 5.10E+02 << 2.32E+02 << 1.08E+02 << 5.57E+01 << 2.96E+01 << 1.00E+01 << 6.00E+00 << 5.00E+00 << 4.80E+00 << 4.70E+00 << 4.60E+00 << 4.50E+00 << 4.50E+00 << 4.50E+00 << 4.50E+00 << 4.50E+00 << 4.53E+00 << 4.55E+00 << 4.60E+00 << 4.65E+00 << 4.70E+00 << 4.75E+00 << 4.80E+00 << 4.85E+00 << 4.90E+00 << 4.95E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 4.95E+00 << 4.85E+00 << 4.50E+00 << 4.00E+00 << 3.30E+00 << 2.70E+00 << 2.00E+00 << 1.33E+00 << 8.50E-01 << 5.40E-01 << 4.00E-01 << 3.40E-01 << 2.80E-01 << 2.40E-01 << 2.00E-01;
    afglModels[2].getPartPressSpecByMolIndex(0)->setY(&auxValues);
    // 3 - O3
    auxValues = QVector<double> () << 2.78E-02 << 2.80E-02 << 2.85E-02 << 3.20E-02 << 3.57E-02 << 4.72E-02 << 5.84E-02 << 7.89E-02 << 1.04E-01 << 1.57E-01 << 2.37E-01 << 3.62E-01 << 5.23E-01 << 7.04E-01 << 8.00E-01 << 9.00E-01 << 1.10E+00 << 1.40E+00 << 1.80E+00 << 2.30E+00 << 2.90E+00 << 3.50E+00 << 3.90E+00 << 4.30E+00 << 4.70E+00 << 5.10E+00 << 5.60E+00 << 6.10E+00 << 6.80E+00 << 7.10E+00 << 7.20E+00 << 6.90E+00 << 5.90E+00 << 4.60E+00 << 3.70E+00 << 2.75E+00 << 1.70E+00 << 1.00E+00 << 5.50E-01 << 3.20E-01 << 2.50E-01 << 2.30E-01 << 5.50E-01 << 8.00E-01 << 8.00E-01 << 4.00E-01 << 2.00E-01 << 5.00E-02 << 5.00E-03 << 5.00E-04;
    afglModels[2].getPartPressSpecByMolIndex(2)->setY(&auxValues);
    // 4 - N2O
    auxValues = QVector<double> () << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.16E-01 << 3.10E-01 << 2.99E-01 << 2.94E-01 << 2.86E-01 << 2.80E-01 << 2.72E-01 << 2.61E-01 << 2.42E-01 << 2.17E-01 << 1.84E-01 << 1.62E-01 << 1.36E-01 << 1.23E-01 << 1.12E-01 << 1.05E-01 << 9.66E-02 << 8.69E-02 << 7.52E-02 << 6.13E-02 << 5.12E-02 << 3.97E-02 << 3.00E-02 << 2.08E-02 << 1.31E-02 << 8.07E-03 << 4.16E-03 << 2.63E-03 << 1.81E-03 << 1.32E-03 << 1.01E-03 << 7.88E-04 << 6.33E-04 << 5.19E-04 << 4.33E-04 << 3.67E-04 << 3.14E-04 << 2.72E-04 << 2.37E-04 << 2.09E-04 << 1.85E-04;
    afglModels[2].getPartPressSpecByMolIndex(3)->setY(&auxValues);
    // 5 - CO
    auxValues = QVector<double> () << 1.50E-01 << 1.45E-01 << 1.40E-01 << 1.35E-01 << 1.31E-01 << 1.30E-01 << 1.29E-01 << 1.25E-01 << 1.19E-01 << 1.09E-01 << 9.96E-02 << 8.96E-02 << 7.81E-02 << 6.37E-02 << 5.03E-02 << 3.94E-02 << 3.07E-02 << 2.49E-02 << 1.97E-02 << 1.55E-02 << 1.33E-02 << 1.23E-02 << 1.23E-02 << 1.31E-02 << 1.40E-02 << 1.50E-02 << 1.60E-02 << 1.71E-02 << 1.85E-02 << 2.00E-02 << 2.15E-02 << 2.33E-02 << 2.62E-02 << 3.06E-02 << 3.80E-02 << 6.25E-02 << 1.48E-01 << 2.93E-01 << 5.59E-01 << 1.08E+00 << 1.90E+00 << 2.96E+00 << 4.53E+00 << 6.86E+00 << 1.05E+01 << 1.71E+01 << 2.47E+01 << 3.36E+01 << 4.15E+01 << 5.00E+01;
    afglModels[2].getPartPressSpecByMolIndex(4)->setY(&auxValues);
    // 6 - CH4
    auxValues = QVector<double> () << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.69E+00 << 1.67E+00 << 1.65E+00 << 1.63E+00 << 1.62E+00 << 1.58E+00 << 1.54E+00 << 1.51E+00 << 1.48E+00 << 1.45E+00 << 1.42E+00 << 1.39E+00 << 1.36E+00 << 1.32E+00 << 1.28E+00 << 1.22E+00 << 1.15E+00 << 1.07E+00 << 9.73E-01 << 8.80E-01 << 7.93E-01 << 7.13E-01 << 6.44E-01 << 5.75E-01 << 5.05E-01 << 4.48E-01 << 3.93E-01 << 3.40E-01 << 2.88E-01 << 2.39E-01 << 1.94E-01 << 1.57E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.40E-01 << 1.30E-01 << 1.20E-01 << 1.10E-01 << 9.50E-02 << 6.00E-02 << 3.00E-02;
    afglModels[2].getPartPressSpecByMolIndex(5)->setY(&auxValues);
    // Other model parameters (Total pressure, Temeprature, Atmosperhic density)
    auxValues = QVector<double> () << 1.02E+03 << 8.97E+02 << 7.90E+02 << 6.94E+02 << 6.08E+02 << 5.31E+02 << 4.63E+02 << 4.02E+02 << 3.47E+02 << 2.99E+02 << 2.57E+02 << 2.20E+02 << 1.88E+02 << 1.61E+02 << 1.38E+02 << 1.18E+02 << 1.01E+02 << 8.61E+01 << 7.36E+01 << 6.28E+01 << 5.37E+01 << 4.58E+01 << 3.91E+01 << 3.34E+01 << 2.86E+01 << 2.44E+01 << 1.65E+01 << 1.11E+01 << 7.56E+00 << 5.18E+00 << 3.60E+00 << 2.53E+00 << 1.80E+00 << 1.29E+00 << 9.40E-01 << 6.83E-01 << 3.62E-01 << 1.88E-01 << 9.50E-02 << 4.70E-02 << 2.22E-02 << 1.03E-02 << 4.56E-03 << 1.98E-03 << 8.77E-04 << 4.07E-04 << 2.00E-04 << 1.06E-04 << 5.98E-05 << 3.60E-05;
    afglModels[2].getTotalPress()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[2].getTotalPress()->setX(&auxValues);
    auxValues = QVector<double> () << 2.72E+02 << 2.69E+02 << 2.65E+02 << 2.62E+02 << 2.56E+02 << 2.50E+02 << 2.44E+02 << 2.38E+02 << 2.32E+02 << 2.26E+02 << 2.20E+02 << 2.19E+02 << 2.19E+02 << 2.18E+02 << 2.18E+02 << 2.17E+02 << 2.17E+02 << 2.16E+02 << 2.16E+02 << 2.15E+02 << 2.15E+02 << 2.15E+02 << 2.15E+02 << 2.15E+02 << 2.15E+02 << 2.15E+02 << 2.16E+02 << 2.17E+02 << 2.20E+02 << 2.28E+02 << 2.36E+02 << 2.43E+02 << 2.51E+02 << 2.59E+02 << 2.65E+02 << 2.66E+02 << 2.61E+02 << 2.51E+02 << 2.41E+02 << 2.31E+02 << 2.20E+02 << 2.10E+02 << 2.00E+02 << 2.00E+02 << 2.08E+02 << 2.19E+02 << 2.37E+02 << 2.60E+02 << 2.93E+02 << 3.33E+02;
    afglModels[2].getTemp()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[2].getTemp()->setX(&auxValues);
    auxValues = QVector<double> () << 2.71E+19 << 2.42E+19 << 2.16E+19 << 1.92E+19 << 1.72E+19 << 1.54E+19 << 1.38E+19 << 1.23E+19 << 1.09E+19 << 9.61E+18 << 8.47E+18 << 7.27E+18 << 6.24E+18 << 5.35E+18 << 4.59E+18 << 3.93E+18 << 3.37E+18 << 2.89E+18 << 2.47E+18 << 2.12E+18 << 1.81E+18 << 1.54E+18 << 1.32E+18 << 1.13E+18 << 9.63E+17 << 8.22E+17 << 5.54E+17 << 3.70E+17 << 2.49E+17 << 1.65E+17 << 1.11E+17 << 7.54E+16 << 5.20E+16 << 3.62E+16 << 2.57E+16 << 1.86E+16 << 1.01E+16 << 5.43E+15 << 2.86E+15 << 1.48E+15 << 7.30E+14 << 3.55E+14 << 1.65E+14 << 7.19E+13 << 3.05E+13 << 1.35E+13 << 6.11E+12 << 2.95E+12 << 1.48E+12 << 7.84E+11;
    afglModels[2].getDensity()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[2].getDensity()->setX(&auxValues);
    afglModels[2].setModelName("Midlatitude winter");


    // 4 - Subarctic summer
    // 1 - H2O
    auxValues = QVector<double> () << 1.19E+04 << 8.70E+03 << 6.75E+03 << 4.82E+03 << 3.38E+03 << 2.22E+03 << 1.33E+03 << 7.97E+02 << 4.00E+02 << 1.30E+02 << 4.24E+01 << 1.33E+01 << 6.00E+00 << 4.45E+00 << 4.00E+00 << 4.00E+00 << 4.00E+00 << 4.05E+00 << 4.30E+00 << 4.50E+00 << 4.60E+00 << 4.70E+00 << 4.80E+00 << 4.83E+00 << 4.85E+00 << 4.90E+00 << 4.95E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 4.95E+00 << 4.85E+00 << 4.50E+00 << 4.00E+00 << 3.30E+00 << 2.70E+00 << 2.00E+00 << 1.33E+00 << 8.50E-01 << 5.40E-01 << 4.00E-01 << 3.40E-01 << 2.80E-01 << 2.40E-01 << 2.00E-01;
    afglModels[3].getPartPressSpecByMolIndex(0)->setY(&auxValues);
    // 3 - O3
    auxValues = QVector<double> () << 2.41E-02 << 2.94E-02 << 3.38E-02 << 3.89E-02 << 4.48E-02 << 5.33E-02 << 6.56E-02 << 7.74E-02 << 9.11E-02 << 1.42E-01 << 1.89E-01 << 3.05E-01 << 4.10E-01 << 5.00E-01 << 6.00E-01 << 7.00E-01 << 8.50E-01 << 1.00E+00 << 1.30E+00 << 1.70E+00 << 2.10E+00 << 2.70E+00 << 3.30E+00 << 3.70E+00 << 4.20E+00 << 4.50E+00 << 5.30E+00 << 5.70E+00 << 6.90E+00 << 7.70E+00 << 7.80E+00 << 7.00E+00 << 5.40E+00 << 4.20E+00 << 3.20E+00 << 2.50E+00 << 1.70E+00 << 1.20E+00 << 8.00E-01 << 4.00E-01 << 2.00E-01 << 1.80E-01 << 6.50E-01 << 9.00E-01 << 8.00E-01 << 4.00E-01 << 2.00E-01 << 5.00E-02 << 5.00E-03 << 5.00E-04;
    afglModels[3].getPartPressSpecByMolIndex(2)->setY(&auxValues);
    // 4 - N2O
    auxValues = QVector<double> () << 3.10E-01 << 3.10E-01 << 3.10E-01 << 3.10E-01 << 3.08E-01 << 3.02E-01 << 2.91E-01 << 2.82E-01 << 2.76E-01 << 2.70E-01 << 2.65E-01 << 2.60E-01 << 2.55E-01 << 2.49E-01 << 2.43E-01 << 2.36E-01 << 2.28E-01 << 2.18E-01 << 2.04E-01 << 1.82E-01 << 1.57E-01 << 1.35E-01 << 1.22E-01 << 1.10E-01 << 9.89E-02 << 8.78E-02 << 7.33E-02 << 5.94E-02 << 4.15E-02 << 3.03E-02 << 1.95E-02 << 1.27E-02 << 9.00E-03 << 6.29E-03 << 4.56E-03 << 2.80E-03 << 1.77E-03 << 1.21E-03 << 8.87E-04 << 6.76E-04 << 5.54E-04 << 4.65E-04 << 3.98E-04 << 3.46E-04 << 3.05E-04 << 2.71E-04 << 2.44E-04 << 2.21E-04 << 2.02E-04 << 1.85E-04;
    afglModels[3].getPartPressSpecByMolIndex(3)->setY(&auxValues);
    // 5 - CO
    auxValues = QVector<double> () << 1.50E-01 << 1.45E-01 << 1.40E-01 << 1.35E-01 << 1.31E-01 << 1.30E-01 << 1.29E-01 << 1.25E-01 << 1.19E-01 << 1.09E-01 << 9.96E-02 << 8.96E-02 << 7.81E-02 << 6.37E-02 << 5.03E-02 << 3.94E-02 << 3.07E-02 << 2.49E-02 << 1.97E-02 << 1.55E-02 << 1.33E-02 << 1.23E-02 << 1.23E-02 << 1.31E-02 << 1.40E-02 << 1.51E-02 << 1.65E-02 << 1.81E-02 << 2.00E-02 << 2.18E-02 << 2.34E-02 << 2.50E-02 << 2.65E-02 << 2.81E-02 << 3.00E-02 << 3.22E-02 << 3.65E-02 << 4.59E-02 << 6.38E-02 << 1.18E-01 << 3.03E-01 << 7.89E-01 << 1.82E+00 << 3.40E+00 << 5.92E+00 << 1.04E+01 << 1.88E+01 << 2.87E+01 << 3.89E+01 << 5.00E+01;
    afglModels[3].getPartPressSpecByMolIndex(4)->setY(&auxValues);
    // 6 - CH4
    auxValues = QVector<double> () << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.69E+00 << 1.67E+00 << 1.65E+00 << 1.63E+00 << 1.62E+00 << 1.58E+00 << 1.54E+00 << 1.51E+00 << 1.47E+00 << 1.43E+00 << 1.39E+00 << 1.34E+00 << 1.29E+00 << 1.23E+00 << 1.16E+00 << 1.07E+00 << 9.90E-01 << 9.17E-01 << 8.57E-01 << 8.01E-01 << 7.48E-01 << 6.96E-01 << 6.44E-01 << 5.89E-01 << 5.24E-01 << 4.51E-01 << 3.71E-01 << 2.99E-01 << 2.45E-01 << 2.00E-01 << 1.66E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.40E-01 << 1.30E-01 << 1.20E-01 << 1.10E-01 << 9.50E-02 << 6.00E-02 << 3.00E-02;
    afglModels[3].getPartPressSpecByMolIndex(5)->setY(&auxValues);
    // Other model parameters (Total pressure, Temeprature, Atmosperhic density)
    auxValues = QVector<double> () << 1.01E+03 << 8.96E+02 << 7.93E+02 << 7.00E+02 << 6.16E+02 << 5.41E+02 << 4.74E+02 << 4.13E+02 << 3.59E+02 << 3.11E+02 << 2.68E+02 << 2.30E+02 << 1.98E+02 << 1.70E+02 << 1.46E+02 << 1.26E+02 << 1.08E+02 << 9.28E+01 << 7.98E+01 << 6.86E+01 << 5.90E+01 << 5.07E+01 << 4.36E+01 << 3.75E+01 << 3.23E+01 << 2.78E+01 << 1.92E+01 << 1.34E+01 << 9.40E+00 << 6.61E+00 << 4.72E+00 << 3.40E+00 << 2.48E+00 << 1.82E+00 << 1.34E+00 << 9.87E-01 << 5.37E-01 << 2.88E-01 << 1.47E-01 << 7.10E-02 << 3.20E-02 << 1.25E-02 << 4.51E-03 << 1.61E-03 << 6.06E-04 << 2.48E-04 << 1.13E-04 << 6.00E-05 << 3.54E-05 << 2.26E-05;
    afglModels[3].getTotalPress()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[3].getTotalPress()->setX(&auxValues);
    auxValues = QVector<double> () << 2.87E+02 << 2.82E+02 << 2.76E+02 << 2.71E+02 << 2.66E+02 << 2.60E+02 << 2.53E+02 << 2.46E+02 << 2.39E+02 << 2.32E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.25E+02 << 2.27E+02 << 2.28E+02 << 2.31E+02 << 2.35E+02 << 2.40E+02 << 2.47E+02 << 2.55E+02 << 2.62E+02 << 2.70E+02 << 2.74E+02 << 2.76E+02 << 2.77E+02 << 2.74E+02 << 2.63E+02 << 2.40E+02 << 2.17E+02 << 1.94E+02 << 1.71E+02 << 1.62E+02 << 1.62E+02 << 1.77E+02 << 1.90E+02 << 2.26E+02 << 2.70E+02 << 3.23E+02 << 3.80E+02;
    afglModels[3].getTemp()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[3].getTemp()->setX(&auxValues);
    auxValues = QVector<double> () << 2.55E+19 << 2.31E+19 << 2.08E+19 << 1.87E+19 << 1.68E+19 << 1.51E+19 << 1.36E+19 << 1.22E+19 << 1.09E+19 << 9.70E+18 << 8.62E+18 << 7.40E+18 << 6.36E+18 << 5.47E+18 << 4.70E+18 << 4.06E+18 << 3.48E+18 << 2.99E+18 << 2.57E+18 << 2.21E+18 << 1.90E+18 << 1.63E+18 << 1.40E+18 << 1.21E+18 << 1.03E+18 << 8.83E+17 << 6.03E+17 << 4.13E+17 << 2.84E+17 << 1.94E+17 << 1.34E+17 << 9.40E+16 << 6.67E+16 << 4.82E+16 << 3.52E+16 << 2.58E+16 << 1.42E+16 << 7.95E+15 << 4.45E+15 << 2.38E+15 << 1.20E+15 << 5.31E+14 << 2.02E+14 << 7.22E+13 << 2.48E+13 << 9.44E+12 << 3.62E+12 << 1.61E+12 << 7.95E+11 << 4.31E+11;
    afglModels[3].getDensity()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[3].getDensity()->setX(&auxValues);
    afglModels[3].setModelName("Subarctic summer");


    // 5 - Subarctic winter
    // 1 - H2O
    auxValues = QVector<double> () << 1.41E+03 << 1.62E+03 << 1.43E+03 << 1.17E+03 << 7.90E+02 << 4.31E+02 << 2.37E+02 << 1.47E+02 << 3.38E+01 << 2.98E+01 << 2.00E+01 << 1.00E+01 << 6.00E+00 << 4.45E+00 << 4.50E+00 << 4.55E+00 << 4.60E+00 << 4.65E+00 << 4.70E+00 << 4.75E+00 << 4.80E+00 << 4.85E+00 << 4.90E+00 << 4.95E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 5.00E+00 << 4.95E+00 << 4.85E+00 << 4.50E+00 << 4.00E+00 << 3.30E+00 << 2.70E+00 << 2.00E+00 << 1.33E+00 << 8.50E-01 << 5.40E-01 << 4.00E-01 << 3.40E-01 << 2.80E-01 << 2.40E-01 << 2.00E-01;
    afglModels[4].getPartPressSpecByMolIndex(0)->setY(&auxValues);
    // 3 - O3
    auxValues = QVector<double> () << 1.80E-02 << 2.07E-02 << 2.34E-02 << 2.77E-02 << 3.25E-02 << 3.80E-02 << 4.45E-02 << 7.25E-02 << 1.04E-01 << 2.10E-01 << 3.00E-01 << 3.50E-01 << 4.00E-01 << 6.50E-01 << 9.00E-01 << 1.20E+00 << 1.50E+00 << 1.90E+00 << 2.45E+00 << 3.10E+00 << 3.70E+00 << 4.00E+00 << 4.20E+00 << 4.50E+00 << 4.60E+00 << 4.70E+00 << 4.90E+00 << 5.40E+00 << 5.90E+00 << 6.20E+00 << 6.25E+00 << 5.90E+00 << 5.10E+00 << 4.10E+00 << 3.00E+00 << 2.60E+00 << 1.60E+00 << 9.50E-01 << 6.50E-01 << 5.00E-01 << 3.30E-01 << 1.30E-01 << 7.50E-01 << 8.00E-01 << 8.00E-01 << 4.00E-01 << 2.00E-01 << 5.00E-02 << 5.00E-03 << 5.00E-04;
    afglModels[4].getPartPressSpecByMolIndex(2)->setY(&auxValues);
    // 4 - N2O
    auxValues = QVector<double> () << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.16E-01 << 3.10E-01 << 2.99E-01 << 2.94E-01 << 2.86E-01 << 2.80E-01 << 2.72E-01 << 2.61E-01 << 2.42E-01 << 2.17E-01 << 1.84E-01 << 1.62E-01 << 1.36E-01 << 1.23E-01 << 1.12E-01 << 1.04E-01 << 9.57E-02 << 8.60E-02 << 7.31E-02 << 5.71E-02 << 4.67E-02 << 3.44E-02 << 2.47E-02 << 1.63E-02 << 1.07E-02 << 7.06E-03 << 3.97E-03 << 2.51E-03 << 1.73E-03 << 1.26E-03 << 9.60E-04 << 7.55E-04 << 6.10E-04 << 5.02E-04 << 4.21E-04 << 3.58E-04 << 3.08E-04 << 2.68E-04 << 2.35E-04 << 2.08E-04 << 1.85E-04;
    afglModels[4].getPartPressSpecByMolIndex(3)->setY(&auxValues);
    // 5 - CO
    auxValues = QVector<double> () << 1.50E-01 << 1.45E-01 << 1.40E-01 << 1.35E-01 << 1.31E-01 << 1.30E-01 << 1.29E-01 << 1.25E-01 << 1.19E-01 << 1.09E-01 << 9.96E-02 << 8.96E-02 << 7.81E-02 << 6.37E-02 << 5.03E-02 << 3.94E-02 << 3.07E-02 << 2.49E-02 << 1.97E-02 << 1.55E-02 << 1.33E-02 << 1.23E-02 << 1.23E-02 << 1.31E-02 << 1.40E-02 << 1.52E-02 << 1.72E-02 << 2.04E-02 << 2.49E-02 << 3.17E-02 << 4.43E-02 << 6.47E-02 << 1.04E-01 << 1.51E-01 << 2.16E-01 << 3.14E-01 << 4.84E-01 << 7.15E-01 << 1.07E+00 << 1.52E+00 << 2.17E+00 << 3.06E+00 << 4.56E+00 << 6.88E+00 << 1.06E+01 << 1.71E+01 << 2.47E+01 << 3.36E+01 << 4.15E+01 << 5.00E+01;
    afglModels[4].getPartPressSpecByMolIndex(4)->setY(&auxValues);
    // 6 - CH4
    auxValues = QVector<double> () << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.69E+00 << 1.67E+00 << 1.65E+00 << 1.63E+00 << 1.62E+00 << 1.58E+00 << 1.54E+00 << 1.51E+00 << 1.47E+00 << 1.43E+00 << 1.39E+00 << 1.34E+00 << 1.29E+00 << 1.23E+00 << 1.16E+00 << 1.08E+00 << 1.01E+00 << 9.56E-01 << 9.01E-01 << 8.48E-01 << 7.96E-01 << 7.45E-01 << 6.94E-01 << 6.43E-01 << 5.88E-01 << 5.24E-01 << 4.51E-01 << 3.71E-01 << 3.00E-01 << 2.45E-01 << 1.98E-01 << 1.59E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.40E-01 << 1.30E-01 << 1.20E-01 << 1.10E-01 << 9.50E-02 << 6.00E-02 << 3.00E-02;
    afglModels[4].getPartPressSpecByMolIndex(5)->setY(&auxValues);
    // Other model parameters (Total pressure, Temeprature, Atmosperhic density)
    auxValues = QVector<double> () << 1.01E+03 << 8.88E+02 << 7.78E+02 << 6.80E+02 << 5.93E+02 << 5.16E+02 << 4.47E+02 << 3.85E+02 << 3.31E+02 << 2.83E+02 << 2.42E+02 << 2.07E+02 << 1.77E+02 << 1.51E+02 << 1.29E+02 << 1.10E+02 << 9.43E+01 << 8.06E+01 << 6.88E+01 << 5.88E+01 << 5.01E+01 << 4.28E+01 << 3.65E+01 << 3.11E+01 << 2.65E+01 << 2.26E+01 << 1.51E+01 << 1.02E+01 << 6.91E+00 << 4.70E+00 << 3.23E+00 << 2.24E+00 << 1.57E+00 << 1.11E+00 << 7.90E-01 << 5.72E-01 << 2.99E-01 << 1.55E-01 << 7.90E-02 << 4.00E-02 << 2.00E-02 << 9.66E-03 << 4.50E-03 << 2.02E-03 << 9.07E-04 << 4.23E-04 << 2.07E-04 << 1.08E-04 << 6.00E-05 << 3.59E-05;
    afglModels[4].getTotalPress()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[4].getTotalPress()->setX(&auxValues);
    auxValues = QVector<double> () << 2.57E+02 << 2.59E+02 << 2.56E+02 << 2.53E+02 << 2.48E+02 << 2.41E+02 << 2.34E+02 << 2.27E+02 << 2.21E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.16E+02 << 2.15E+02 << 2.15E+02 << 2.14E+02 << 2.14E+02 << 2.13E+02 << 2.12E+02 << 2.12E+02 << 2.11E+02 << 2.14E+02 << 2.16E+02 << 2.19E+02 << 2.22E+02 << 2.29E+02 << 2.35E+02 << 2.41E+02 << 2.47E+02 << 2.53E+02 << 2.59E+02 << 2.59E+02 << 2.51E+02 << 2.48E+02 << 2.45E+02 << 2.35E+02 << 2.24E+02 << 2.13E+02 << 2.02E+02 << 2.11E+02 << 2.19E+02 << 2.34E+02 << 2.53E+02 << 2.89E+02 << 3.33E+02;
    afglModels[4].getTemp()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[4].getTemp()->setX(&auxValues);
    auxValues = QVector<double> () << 2.86E+19 << 2.48E+19 << 2.20E+19 << 1.95E+19 << 1.74E+19 << 1.55E+19 << 1.38E+19 << 1.23E+19 << 1.09E+19 << 9.44E+18 << 8.07E+18 << 6.90E+18 << 5.89E+18 << 5.04E+18 << 4.31E+18 << 3.68E+18 << 3.16E+18 << 2.70E+18 << 2.32E+18 << 1.98E+18 << 1.70E+18 << 1.45E+18 << 1.24E+18 << 1.06E+18 << 9.07E+17 << 7.74E+17 << 5.13E+17 << 3.42E+17 << 2.29E+17 << 1.53E+17 << 1.03E+17 << 6.93E+16 << 4.73E+16 << 3.27E+16 << 2.26E+16 << 1.60E+16 << 8.36E+15 << 4.48E+15 << 2.31E+15 << 1.18E+15 << 6.18E+14 << 3.13E+14 << 1.53E+14 << 7.24E+13 << 3.12E+13 << 1.40E+13 << 6.41E+12 << 3.10E+12 << 1.51E+12 << 7.81E+11;
    afglModels[4].getDensity()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[4].getDensity()->setX(&auxValues);
    afglModels[4].setModelName("Subarctic winter");


    // 6 - 	US Standard
    // 1 - H2O
    auxValues = QVector<double> () << 7.75E+03 << 6.07E+03 << 4.63E+03 << 3.18E+03 << 2.16E+03 << 1.40E+03 << 9.25E+02 << 5.72E+02 << 3.67E+02 << 1.58E+02 << 7.00E+01 << 3.61E+01 << 1.91E+01 << 1.09E+01 << 5.93E+00 << 5.00E+00 << 3.95E+00 << 3.85E+00 << 3.83E+00 << 3.85E+00 << 3.90E+00 << 3.98E+00 << 4.07E+00 << 4.20E+00 << 4.30E+00 << 4.43E+00 << 4.58E+00 << 4.73E+00 << 4.83E+00 << 4.90E+00 << 4.95E+00 << 5.03E+00 << 5.15E+00 << 5.23E+00 << 5.25E+00 << 5.23E+00 << 5.10E+00 << 4.75E+00 << 4.20E+00 << 3.50E+00 << 2.83E+00 << 2.05E+00 << 1.33E+00 << 8.50E-01 << 5.40E-01 << 4.00E-01 << 3.40E-01 << 2.80E-01 << 2.40E-01 << 2.00E-01;
    afglModels[5].getPartPressSpecByMolIndex(0)->setY(&auxValues);
    // 3 - O3
    auxValues = QVector<double> () << 2.66E-02 << 2.93E-02 << 3.24E-02 << 3.32E-02 << 3.39E-02 << 3.77E-02 << 4.11E-02 << 5.01E-02 << 5.97E-02 << 9.17E-02 << 1.31E-01 << 2.15E-01 << 3.10E-01 << 3.85E-01 << 5.03E-01 << 6.51E-01 << 8.70E-01 << 1.19E+00 << 1.59E+00 << 2.03E+00 << 2.58E+00 << 3.03E+00 << 3.65E+00 << 4.17E+00 << 4.63E+00 << 5.12E+00 << 5.80E+00 << 6.55E+00 << 7.37E+00 << 7.84E+00 << 7.80E+00 << 7.30E+00 << 6.20E+00 << 5.25E+00 << 4.10E+00 << 3.10E+00 << 1.80E+00 << 1.10E+00 << 7.00E-01 << 3.00E-01 << 2.50E-01 << 3.00E-01 << 5.00E-01 << 7.00E-01 << 7.00E-01 << 4.00E-01 << 2.00E-01 << 5.00E-02 << 5.00E-03 << 5.00E-04;
    afglModels[5].getPartPressSpecByMolIndex(2)->setY(&auxValues);
    // 4 - N2O
    auxValues = QVector<double> () << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.20E-01 << 3.18E-01 << 3.14E-01 << 3.10E-01 << 3.05E-01 << 3.00E-01 << 2.94E-01 << 2.88E-01 << 2.78E-01 << 2.67E-01 << 2.53E-01 << 2.37E-01 << 2.19E-01 << 2.05E-01 << 1.97E-01 << 1.88E-01 << 1.76E-01 << 1.59E-01 << 1.42E-01 << 1.17E-01 << 9.28E-02 << 6.69E-02 << 4.51E-02 << 2.75E-02 << 1.59E-02 << 9.38E-03 << 4.75E-03 << 3.00E-03 << 2.07E-03 << 1.51E-03 << 1.15E-03 << 8.89E-04 << 7.06E-04 << 5.72E-04 << 4.71E-04 << 3.93E-04 << 3.32E-04 << 2.84E-04 << 2.44E-04 << 2.12E-04 << 1.85E-04;
    afglModels[5].getPartPressSpecByMolIndex(3)->setY(&auxValues);
    // 5 - CO
    auxValues = QVector<double> () << 1.50E-01 << 1.45E-01 << 1.40E-01 << 1.35E-01 << 1.31E-01 << 1.30E-01 << 1.29E-01 << 1.25E-01 << 1.19E-01 << 1.09E-01 << 9.96E-02 << 8.96E-02 << 7.81E-02 << 6.37E-02 << 5.03E-02 << 3.94E-02 << 3.07E-02 << 2.49E-02 << 1.97E-02 << 1.55E-02 << 1.33E-02 << 1.23E-02 << 1.23E-02 << 1.31E-02 << 1.40E-02 << 1.50E-02 << 1.60E-02 << 1.71E-02 << 1.85E-02 << 2.01E-02 << 2.22E-02 << 2.50E-02 << 2.82E-02 << 3.24E-02 << 3.72E-02 << 4.60E-02 << 6.64E-02 << 1.07E-01 << 1.86E-01 << 3.06E-01 << 6.38E-01 << 1.50E+00 << 3.24E+00 << 5.84E+00 << 1.01E+01 << 1.69E+01 << 2.47E+01 << 3.36E+01 << 4.15E+01 << 5.00E+01;
    afglModels[5].getPartPressSpecByMolIndex(4)->setY(&auxValues);
    // 6 - CH4
    auxValues = QVector<double> () << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.70E+00 << 1.69E+00 << 1.69E+00 << 1.68E+00 << 1.66E+00 << 1.65E+00 << 1.63E+00 << 1.61E+00 << 1.58E+00 << 1.55E+00 << 1.52E+00 << 1.48E+00 << 1.42E+00 << 1.36E+00 << 1.27E+00 << 1.19E+00 << 1.12E+00 << 1.06E+00 << 9.87E-01 << 9.14E-01 << 8.30E-01 << 7.46E-01 << 6.62E-01 << 5.64E-01 << 4.61E-01 << 3.63E-01 << 2.77E-01 << 2.10E-01 << 1.65E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.50E-01 << 1.40E-01 << 1.30E-01 << 1.20E-01 << 1.10E-01 << 9.50E-02 << 6.00E-02 << 3.00E-02;
    afglModels[5].getPartPressSpecByMolIndex(5)->setY(&auxValues);
    // Other model parameters (Total pressure, Temeprature, Atmosperhic density)
    auxValues = QVector<double> () << 1.01E+03 << 8.99E+02 << 7.95E+02 << 7.01E+02 << 6.17E+02 << 5.41E+02 << 4.72E+02 << 4.11E+02 << 3.57E+02 << 3.08E+02 << 2.65E+02 << 2.27E+02 << 1.94E+02 << 1.66E+02 << 1.42E+02 << 1.21E+02 << 1.04E+02 << 8.85E+01 << 7.57E+01 << 6.47E+01 << 5.53E+01 << 4.73E+01 << 4.05E+01 << 3.47E+01 << 2.97E+01 << 2.55E+01 << 1.74E+01 << 1.20E+01 << 8.01E+00 << 5.75E+00 << 4.15E+00 << 2.87E+00 << 2.06E+00 << 1.49E+00 << 1.09E+00 << 7.98E-01 << 4.25E-01 << 2.19E-01 << 1.09E-01 << 5.22E-02 << 2.40E-02 << 1.05E-02 << 4.46E-03 << 1.84E-03 << 7.60E-04 << 3.20E-04 << 1.45E-04 << 7.10E-05 << 4.01E-05 << 2.54E-05;
    afglModels[5].getTotalPress()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[5].getTotalPress()->setX(&auxValues);
    auxValues = QVector<double> () << 2.88E+02 << 2.82E+02 << 2.75E+02 << 2.69E+02 << 2.62E+02 << 2.56E+02 << 2.49E+02 << 2.43E+02 << 2.36E+02 << 2.30E+02 << 2.23E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.17E+02 << 2.18E+02 << 2.19E+02 << 2.20E+02 << 2.21E+02 << 2.22E+02 << 2.24E+02 << 2.27E+02 << 2.30E+02 << 2.37E+02 << 2.43E+02 << 2.50E+02 << 2.57E+02 << 2.64E+02 << 2.71E+02 << 2.71E+02 << 2.61E+02 << 2.47E+02 << 2.33E+02 << 2.20E+02 << 2.08E+02 << 1.99E+02 << 1.89E+02 << 1.87E+02 << 1.88E+02 << 1.95E+02 << 2.09E+02 << 2.40E+02 << 3.00E+02 << 3.60E+02;
    afglModels[5].getTemp()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[5].getTemp()->setX(&auxValues);
    auxValues = QVector<double> () << 2.55E+19 << 2.31E+19 << 2.09E+19 << 1.89E+19 << 1.70E+19 << 1.53E+19 << 1.37E+19 << 1.23E+19 << 1.09E+19 << 9.72E+18 << 8.60E+18 << 7.59E+18 << 6.49E+18 << 5.55E+18 << 4.74E+18 << 4.05E+18 << 3.46E+18 << 2.96E+18 << 2.53E+18 << 2.16E+18 << 1.85E+18 << 1.58E+18 << 1.34E+18 << 1.14E+18 << 9.77E+17 << 8.34E+17 << 5.64E+17 << 3.83E+17 << 2.52E+17 << 1.76E+17 << 1.24E+17 << 8.31E+16 << 5.80E+16 << 4.09E+16 << 2.92E+16 << 2.14E+16 << 1.18E+16 << 6.43E+15 << 3.39E+15 << 1.72E+15 << 8.35E+14 << 3.83E+14 << 1.71E+14 << 7.14E+13 << 2.92E+13 << 1.19E+13 << 5.03E+12 << 2.14E+12 << 9.69E+11 << 5.11E+11;
    afglModels[5].getDensity()->setY(&auxValues);
    auxValues = QVector<double> () << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 17 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 27.5 << 30 << 32.5 << 35 << 37.5 << 40 << 42.5 << 45 << 47.5 << 50 << 55 << 60 << 65 << 70 << 75 << 80 << 85 << 90 << 95 << 100 << 105 << 110 << 115 << 120;
    afglModels[5].getDensity()->setX(&auxValues);
    afglModels[5].setModelName("US Standard");

    // Converting total pressure values in the models into the dfault atm units
    pressConvert.setPressUnits(1);
    for (i = 0; i < afglModels.count(); i++)
    {
        for (j = 0; j < afglModels[i].getTotalPress()->getY()->count(); j++)
        {
            afglModels[i].getTotalPress()->setYAt(j, pressConvert.fromCurrentToDefaultPressUnits(afglModels[i].getTotalPress()->getYAt(j)));
        }
    }
}

// Number of the active atmospheric model
int Atmosphere::getAtiveAtmModelNum() { return this->activeAtmModelNumber; }
void Atmosphere::setActiveAtmModelNum(int newActiveAtmModelNum) { this->activeAtmModelNumber = newActiveAtmModelNum; }
